************************************************************************
*                                                                      *
      subroutine usrtally(ncol)
*                                                                      *
*        sample subroutine for user defined tally.                     *
*                                                                      *
*----------------------------------------------------------------------*
*                                                                      *
*       ncol = 1 : start of calculation                                *
*              2 : end of calculation                                  *
*              3 : end of a batch                                      *
*              4 : source                                              *
*              5 : detection of geometry error                         *
*              6 : recovery of geometry error                          *
*              7 : termination by geometry error                       *
*              8 : termination by weight cut-off                       *
*              9 : termination by time cut-off                         *
*             10 : geometry boundary crossing                          *
*             11 : termination by energy cut-off                       *
*             12 : termination by escape or leakage                    *
*             13 : (n,x) reaction                                      *
*             14 : (n,n'x) reaction                                    *
*             15 : sequential transport only for tally                 *
*                                                                      *
*----------------------------------------------------------------------*
*                                                                      *
*        In the distributed-memory parallel computing,                 *
*          npe : total number of used Processor Elements               *
*          me : ID number of each processor                            *
*                                                                      *
*----------------------------------------------------------------------*
*                                                                      *
*        In the shared memory parallel computing,                      *
*          ipomp : ID number of each core                              *
*          npomp : total number of used cores                          *
*                                                                      *
*----------------------------------------------------------------------*
*                                                                      *
*        nocas : current event number in this batch                    :
*        nobch : current batch number                                  *
*        rcasc : real number of NOCAS+maxcas*(NOBCH-1)                 *
*        rsouin : sum of the weight of source particle                 *
*                                                                      *
*----------------------------------------------------------------------*
*                                                                      *
*        no : cascade id in this event                                 *
*        idmn(mat) : material id                                       *
*        ityp : particle type                                          *
*        ktyp : particle kf-code                                       *
*        jtyp : charge number of the particle                          *
*        mtyp : baryon number of the particle                          *
*        rtyp : rest mass of the particle (MeV)                        *
*        oldwt : weight of the particle at (x,y,z)                     *
*        qs : dE/dx of electron at (x,y,z)                             *
*                                                                      *
*----------------------------------------------------------------------*
*                                                                      *
*        iblz1 : cell id at (x,y,z)                                    *
*        iblz2 : cell id after crossing                                *
*        ilev1 : level structure id of the cell at (x,y,z)             *
*          ilat1(i,j)                                                  *
*        ilev2 : level structure id of the cell after crossing         *
*          ilat2(i,j)                                                  *
*        costha : cosine of theta on surface crossing                  *
*        uang(1) : x of position(?) at surface crossing                *
*        uang(2) : y of position(?) at surface crossing                *
*        uang(3) : z of position(?) at surface crossing                *
*        nsurf : surface number                                        *
*                                                                      *
*----------------------------------------------------------------------*
*                                                                      *
*        name(no) : collision number of the particle                   *
*        ncnt(1,no) : values of counter 1                              *
*        ncnt(2,no) : values of counter 2                              *
*        ncnt(3,no) : values of counter 3                              *
*        wt(no) : weight of the particle at (xc,yc,zc)                 *
*        u(no) : x, y, z-components of unit vector of                  *
*        v(no) :momentum of the particle                               *
*        w(no) :                                                       *
*        e(no) : energy of the particle at (x,y,z) (MeV)               *
*        t(no) : time of the particle at (x,y,z) (nsec)                *
*        x(no) : x, y, z-position coordinates of                       *
*        y(no) :the preceding event point (cm)                         *
*        z(no) :                                                       *
*        ec(no) : energy of the particle at (xc,yc,zc) (MeV)           *
*        tc(no) : time of the particle at (xc,yc,zc) (nsec)            *
*        xc(no) : x, y, z-position coordinates of                      *
*        yc(no) :the particle (cm)                                     *
*        zc(no) :                                                      *
*        spx(no) : x, y, z-components of unit vector of                *
*        spy(no) :spin direction of the particle                       *
*        spz(no) :                                                     *
*        nzst(no)                                                      *
*                                                                      *
*----------------------------------------------------------------------*
*                                                                      *
*        nclsts : the number of produced particle and nucleus          *
*        mathz : Z number of the mother nucleus                        *
*        mathn : N number of the mother nucleus                        *
*        jcoll : reaction type id1                                     *
*        kcoll : reaction type id2                                     *
*                                                                      *
*----------------------------------------------------------------------*
*        jcoll reaction type identifier                                *
*----------------------------------------------------------------------*
*                                                                      *
*        jcoll : =  0, nothing happen                                  *
*                =  1, Hydrogen collisions                             *
*                =  2, Particle Decays                                 *
*                =  3, Elastic collisions                              *
*                =  4, High Energy Nuclear collisions                  *
*                =  5, Heavy Ion reactions                             *
*                =  6, Neutron reactions by data                       *
*                =  7, Photon reactions by data                        *
*                =  8, Electron reactions by data                      *
*                =  9, Proton reactions by data                        *
*                = 10, Neutron event mode                              *
*                = 11, Delta Ray production                            *
*                = 12, Muon interaction                                *
*                = 13, Photon by EGS5                                  *
*                = 14, Electron by EGS5                                *
*                                                                      *
*----------------------------------------------------------------------*
*        kcoll reaction type identifier                                *
*----------------------------------------------------------------------*
*                                                                      *
*        kcoll : =  0, normal                                          *
*                =  1, high energy fission                             *
*                =  2, high energy absorption                          *
*                =  3, low energy n elastic                            *
*                =  4, low energy n non-elastic                        *
*                =  5, low energy n fission                            *
*                =  6, low energy n absorption                         *
*                                                                      *
*----------------------------------------------------------------------*
*                                                                      *
*        nclsts   : total number of out going particles and nuclei     *
*                                                                      *
*        iclusts(nclsts)                                               *
*                                                                      *
*                i = 0, nucleus                                        *
*                  = 1, proton                                         *
*                  = 2, neutron                                        *
*                  = 3, pion                                           *
*                  = 4, photon                                         *
*                  = 5, kaon                                           *
*                  = 6, muon                                           *
*                  = 7, others                                         *
*                                                                      *
*        jclusts(i,nclsts)                                             *
*                                                                      *
*                i = 0, angular momentum                               *
*                  = 1, proton number                                  *
*                  = 2, neutron number                                 *
*                  = 3, ip, see below                                  *
*                  = 4, status of the particle 0: real, <0 : dead      *
*                  = 5, charge                                         *
*                  = 6, baryon number                                  *
*                  = 7, kf code                                        *
*                  = 8, isomer level (0: Ground, 1,2: 1st, 2nd isomer) *
*                                                                      *
*        qclusts(i,nclsts)                                             *
*                                                                      *
*                i = 0, impact parameter                               *
*                  = 1, px (GeV/c)                                     *
*                  = 2, py (GeV/c)                                     *
*                  = 3, pz (GeV/c)                                     *
*                  = 4, etot = sqrt( p**2 + rm**2 ) (GeV)              *
*                  = 5, rest mass (GeV)                                *
*                  = 6, excitation energy (MeV)                        *
*                  = 7, kinetic energy (MeV)                           *
*                  = 8, weight                                         *
*                  = 9, time                                           *
*                  = 10, x                                             *
*                  = 11, y                                             *
*                  = 12, z                                             *
*                                                                      *
*        jcount(i,nclsts)                                              *
*                                                                      *
************************************************************************
*                                                                      *
*        kf code table                                                 *
*                                                                      *
*           kf-code: ityp :  description                               *
*                                                                      *
*             2212 :   1  :  proton                                    *
*             2112 :   2  :  neutron                                   *
*              211 :   3  :  pion (+)                                  *
*              111 :   4  :  pion (0)                                  *
*             -211 :   5  :  pion (-)                                  *
*              -13 :   6  :  muon (+)                                  *
*               13 :   7  :  muon (-)                                  *
*              321 :   8  :  kaon (+)                                  *
*              311 :   9  :  kaon (0)                                  *
*             -321 :  10  :  kaon (-)                                  *
*                                                                      *
*               11 :  12  :  electron                                  *
*              -11 :  13  :  positron                                  *
*               22 :  14  :  photon                                    *
*          1000002 :  15  :  deuteron                                  *
*          1000003 :  16  :  triton                                    *
*          2000003 :  17  :  3he                                       *
*          2000004 :  18  :  alpha                                     *
*      Z*1000000+A :  19  :  nucleus                                   *
*                                                                      *
*           kf-code of the other transport particles (ityp=11)         *
*               12 :         nu_e                                      *
*               14 :         nu_mu                                     *
*              221 :         eta                                       *
*              331 :         eta'                                      *
*             -311 :         k0bar                                     *
*            -2112 :         nbar                                      *
*            -2212 :         pbar                                      *
*             3122 :         Lanbda0                                   *
*             3222 :         Sigma+                                    *
*             3212 :         Sigma0                                    *
*             3112 :         Sigma-                                    *
*             3322 :         Xi0                                       *
*             3312 :         Xi-                                       *
*             3334 :         Omega-                                    *
*                                                                      *
************************************************************************

      use MMBANKMOD !FURUTA
      use usrtalmod

      implicit real*8 (a-h,o-z)

*-----------------------------------------------------------------------

      include 'param00.inc'
      include 'param.inc'

*-----------------------------------------------------------------------

      common /mpi00/ npe, me
      common /ipomp0/ipomp,npomp
!$OMP THREADPRIVATE(/ipomp0/)

*-----------------------------------------------------------------------

      common /cusrtally/ iusrtally, iudtf(50)
      common /cudtpara/ udtpara(0:9)

*-----------------------------------------------------------------------

      common /jcomon/ nabov,nobch,nocas,nomax
!$OMP THREADPRIVATE(/jcomon/)
      common /rcomon/ rcasc

      common /icomon/ no,mat,ityp,ktyp,jtyp,mtyp,rtyp
!$OMP THREADPRIVATE(/icomon/)
      common /wtsave/ oldwt
!$OMP THREADPRIVATE(/wtsave/)
      common /tlgeom/ iblz1,iblz2
!$OMP THREADPRIVATE(/tlgeom/)
      common /tlglat/ ilev1,ilev2,ilat1(5,10),ilat2(5,10)
!$OMP THREADPRIVATE(/tlglat/)
      common /tlcost/ costha, uang(3), nsurf
!$OMP THREADPRIVATE(/tlcost/)
      common /celdb/  idsn(kvlmax), idtn(kvlmax)

cKN 2012/10/31
      common /elect/  uint(3), qs, qo, eint, delc, am, qsex,
     &                nq, ns, n1, noz, mtel
!$OMP THREADPRIVATE(/elect/)

      common /mathzn/ mathz, mathn, jcoll, kcoll
!$OMP THREADPRIVATE(/mathzn/)

      common /clustt/ nclsts, iclusts(nnn)
!$OMP THREADPRIVATE(/clustt/)
      common /clustw/ jclusts(0:8,nnn),  qclusts(0:12,nnn)
!$OMP THREADPRIVATE(/clustw/)
      common /cntcls/ jcount(3,nnn)
!$OMP THREADPRIVATE(/cntcls/)

      common /randtp/ rijk,rans,ranb
!$OMP THREADPRIVATE(/randtp/)

      common /taliin/ rsouin, nzztin, nrgnin
      common /kmat1d/ idmn(0:kvlmax), idnm(kvmmax)

      common /dedxfac/ dedxfd
!$OMP THREADPRIVATE(/dedxfac/)

      integer iegsemi, iegsout
      common /egsemi/ iegsemi, iegsout
      real*8 edep
      common/EPCONTedep/ edep

      integer nudtvar
      common /cnudtvar/ nudtvar

      save totaldep ! total deposition energy used in this routine

      io = iudtf(1)

*-----------------------------------------------------------------------
*     similar to [t-deposit] tally
*-----------------------------------------------------------------------

      if( ncol .eq. 1 ) then ! Start of calculation
       totaldep=0.0
       write(io,'(''  #history    #batch      ncol   kf-code   cell-ID''
     & ,''  E_dep(MeV)      Weight'',
     & ''  E_old(MeV)   X_old(cm)   Y_old(cm)   Z_old(cm)'',
     & ''  E_new(MeV)   X_new(cm)   Y_new(cm)   Z_new(cm)'')')
       return
      end if

      if( ncol .eq. 2 ) then ! End of calculation
       write(io,'(''Total Deposition Energy (MeV/source)='',es12.4)')
     & totaldep/rsouin
       return
      endif

      if(ncol.ge.5) then ! timing for depositing energy
       ein=e(no)   ! current particle energy 
       ecc=ec(no)  ! particle energy before this event
       if((ityp.eq.12.or.ityp.eq.13.or.ityp.eq.14).and.iegsemi.ne.0)then  ! EGS mode
        tlngth = sqrt( ( xc(no) - x(no) )**2
     &               + ( yc(no) - y(no) )**2
     &               + ( zc(no) - z(no) )**2 )
        call egs5edxde(ecc,tlngth,ein,mat,ityp)
        if(ncol.eq.9 .or. ncol.eq.11) then
         ecc = 0d0
         if( ityp.eq.12 .or. ityp.eq.13 ) ein = ein + edep
        endif
       else ! original PHITS mode
        ecc = ein - dedxfd * ( ein - ecc ) ! modification factor for electron
       endif
       if(ein.ne.ecc) then
        edepo=ein-ecc ! energy changed
        write(io,'(5i10,20es12.4)') nocas,nobch,ncol,ktyp,iblz1,edepo,
     &  oldwt,e(no),x(no),y(no),z(no),ecc,xc(no),yc(no),zc(no)
        totaldep=totaldep+edepo*oldwt
       endif
      endif

      if((ncol.eq.13.or.ncol.eq.14).and.mat.gt.0.and.nclsts.gt.0) then ! dead particle is produced by a reaction
       do j = 1, nclsts
        if( jclusts(4,j) .lt. 0 .and. jclusts(5,j) .ne. 0 ) then
         ein = qclusts(7,j)
         ecc = 0.0d0
         edepo=ein
         kfd = jclusts(7,j)
         xd  = qclusts(10,j)
         yd  = qclusts(11,j)
         zd  = qclusts(12,j)
         write(io,'(5i10,20es12.4)') nocas,nobch,ncol,kfd,iblz1,edepo,
     &   oldwt,ein,xd,yd,zd,ecc,xd,yd,zd
         totaldep=totaldep+edepo*oldwt
        endif
       enddo
       return
      endif

      return
      end
